#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include <stdio.h>
#include <string.h>

#include "lcd.h"
#include "key.h"
#include "fnd.h"


extern volatile char key_flag;
extern volatile unsigned char fnd_buf[4], dot_buf[4];
extern volatile char fnd_flag, dot_flag;

char user_number[4], password[4];
volatile unsigned char key = 0, user_set;
volatile long save_money;


void mcu_init(void)
{
	lcd_init();
	key_init();
	fnd_init();

	sei();
}

void start_bankbook(void)
{
	fnd_flag = dot_flag = 0;

	lcd_gotoxy(0, 0);
	lcd_string(" Digit Bankbook ");
	lcd_gotoxy(0, 1);
	lcd_string(" Init...        ");

	_delay_ms(3000);

	user_set = 0;
	save_money = 0;
}

void init_screen(void)
{
	lcd_gotoxy(0, 0);
	lcd_string("1.Cash Service  ");
	lcd_gotoxy(0, 1);
	lcd_string("2.User Setting  ");
}

enum {
	USER_MODE = 1,
	PASS_MODE,
	CHECK_MODE,
	CASH_USER,
	CASH_PASS
};

void input_number(char x, char y, char *str, char mode)
{
	unsigned char pos;

	for( int i = 0; i < 4; i++ ) {
		if( dot_flag )	dot_buf[i] = 0;
		if( (mode == USER_MODE) || (mode == CASH_USER) )	fnd_buf[i] = 0x0F;
		str[i] = 0x00;
	}

	pos = 0;

	while( 1 ) {
		key = getkey(key);
		if( key_flag ) {
			switch( key ) {
				case KEY_0 :
				case KEY_1 :
				case KEY_2 :
				case KEY_3 :
				case KEY_4 :
				case KEY_5 :
				case KEY_6 :
				case KEY_7 :
				case KEY_8 :
				case KEY_9 :
					if( pos == 4 ) {
						for( int i = 0; i < 4; i++ ) {
							if( (mode == USER_MODE) || (mode == CASH_USER) )	fnd_buf[i] = 0x0F;
							else									dot_buf[i] = 0;
							str[i] = 0x0F;
						}

						pos = 0;

						lcd_gotoxy(pos + x, y);
						lcd_string("____");
					}

					if( pos < 4 ) {
						if( (mode == USER_MODE) || (mode == CASH_USER) )	fnd_buf[pos] = key - '0';
						else									dot_buf[pos] = 1;
						str[pos] = key;

						lcd_gotoxy(pos + x, y);
						if( (mode == PASS_MODE) || (mode == CHECK_MODE) )	lcd_data_write('-');
						else									lcd_data_write(key);

						pos++;

						if( (pos == 4) && (mode >= CHECK_MODE) )	return;
					}
					break;

				case KEY_ENTER :
					return;

				case KEY_CANCEL :
					pos--;
					if( (mode == USER_MODE) || (mode == CASH_USER) )	fnd_buf[pos] = 0x0F;
					else									dot_buf[pos] = 0;
					str[pos] = 0x00;

					lcd_gotoxy(pos + x, y);
					lcd_data_write('_');
					break;
			}
		}
	}
}

void user_setting(void)
{
	char tbuf[4], STATE;
	int i;

	fnd_flag = 1;
	lcd_gotoxy(0, 0);
	lcd_string("  User Number   ");
	lcd_gotoxy(0, 1);
	lcd_string("      ____      ");
	input_number(6, 1, user_number, USER_MODE);

	dot_flag = 1;
	lcd_gotoxy(0, 0);
	lcd_string("    Password    ");
	lcd_gotoxy(0, 1);
	lcd_string("      ____      ");
	input_number(6, 1, password, PASS_MODE);

	STATE = 1;
	do {
		lcd_gotoxy(0, 0);
		lcd_string(" Check Password ");
		lcd_gotoxy(0, 1);
		lcd_string("      ____      ");
		input_number(6, 1, tbuf, CHECK_MODE);

		for( i = 0; i < 4; i++ ) {
			if( tbuf[i] != password[i] ) {
				lcd_gotoxy(0, 1);
				lcd_string(" Password Error ");
				
				for( int j = 0; j < 3; j++ ) {
					lcd_gotoxy(0, 0);
					lcd_string(" Error!!!!      ");
					_delay_ms(500);
					lcd_gotoxy(0, 0);
					lcd_string("                ");
					_delay_ms(500);
				}

				break;
			}
		}
		
		if( i == 4 ) {
			lcd_gotoxy(0, 0);
			lcd_string("  User Number   ");
			lcd_gotoxy(0, 1);
			lcd_string("      XXXX      ");
			user_set = 1;

			_delay_ms(3000);

			STATE = 0;
		}
	} while( STATE );

	fnd_flag = dot_flag = 0;
}

void new_user_setting(void)
{
////////////////////////////////////////////////////////////////////////////////
// [ 3] ο User Number 
////////////////////////////////////////////////////////////////////////////////	















}

void id_check(void)
{
	char user_buf[4], pass_buf[4], STATE;
	int i;

	fnd_flag = 1;

	STATE = 1;
	do {
////////////////////////////////////////////////////////////////////////////////
// [ 1] Cash Service Mode ID Check
////////////////////////////////////////////////////////////////////////////////	




























	} while( STATE );

	fnd_flag = 0;
}

long input_money(void)
{
	long money = 0, num;
	int pos = 0;
	char buf[10];

	while( 1 ) {
		key = getkey(key);
		if( key_flag ) {
			switch( key ) {
				case KEY_0 :
				case KEY_1 :
				case KEY_2 :
				case KEY_3 :
				case KEY_4 :
				case KEY_5 :
				case KEY_6 :
				case KEY_7 :
				case KEY_8 :
				case KEY_9 :
					if( pos < 6 ) {
						if( (pos == 0) && (key == KEY_0) );
						else {
							lcd_gotoxy(pos + 3, 1);
							lcd_data_write(key);

							buf[pos++] = key - '0';
						}
					}
					break;

				case KEY_MAN :
					if( pos <= 2 ) {
						if( pos == 1 )	money = buf[0];
						else			money = buf[0] * 10 + buf[1];
						money *= 10000;

						return money;
					}
					break;

				case KEY_WON :
					if( pos ) {
						money = 0;
						for( int i = 0; i < pos; i++ ) {
							num = 1;
							for( int j = 0; j < i; j++ )	num *= 10;
							money += buf[pos - i - 1] * num;
						}

						return money;
					}
					break;

				case KEY_CANCEL :
					lcd_gotoxy(0, 1);
					lcd_string(" :              ");
					pos = 0;
					break;
			}
		}
	}	
}

#define IN_MONEY		1
#define OUT_MONEY		2

void inout_mode(void)
{
	long money;
	char STATE, error, mode;

	lcd_gotoxy(0, 0);
	lcd_string("1.Input Money   ");
	lcd_gotoxy(0, 1);
	lcd_string("2.Output Money  ");

	while( 1 ) {
		key = getkey(key);
		if( key_flag ) {
			if( (key == KEY_1) || (key == KEY_2) ) {
				if( key == KEY_1 )	mode = IN_MONEY;
				else				mode = OUT_MONEY;

				error = 0;
				STATE = 1;
				do {
					lcd_gotoxy(0, 0);
					if( mode == IN_MONEY )	lcd_string(" Input Money    ");
					else				lcd_string(" Output Money    ");
					lcd_gotoxy(0, 1);
					lcd_string(" :              ");

					money = input_money();

					lcd_gotoxy(0, 0);
					if( mode == IN_MONEY )	printf(" Input : %6ld ", money);
					else				printf(" Output : %6ld", money);
					lcd_gotoxy(0, 1);
					lcd_string(" Please ENTER!  ");

					while( (key = getkey(key)) != KEY_ENTER );

					if( mode == IN_MONEY ) {
						if( (save_money + money) > 9990000 )	error = 1;
						else	save_money += money;
					}
					else {
						if( (save_money - money) < 0 )		error = 1;
						else	save_money -= money;
					}

					if( error ) {
						lcd_gotoxy(0, 1);
						if( mode == IN_MONEY )	lcd_string(" Balance Excess ");
						else				lcd_string(" Balance Lack   ");

						for( int i = 0; i < 3; i++ ) {
							lcd_gotoxy(0, 0);
							lcd_string(" Error!!!!      ");
							_delay_ms(500);
							lcd_gotoxy(0, 0);
							lcd_string("                ");
							_delay_ms(500);
						}
					}
					else STATE = 0;
				} while( STATE );

				lcd_gotoxy(0, 0);
				lcd_string("Balance Inquiry ");
				lcd_gotoxy(0, 1);
				printf("  %7ld Won   ", save_money);

////////////////////////////////////////////////////////////////////////////////
// [ 2] FND  ܰ ǥ
////////////////////////////////////////////////////////////////////////////////















				_delay_ms(3000);

				fnd_flag = 0;

				return;
			}
		}
	}
}

void account_mode(void)
{
	lcd_gotoxy(0, 0);
	lcd_string("Balance Inquiry ");
	lcd_gotoxy(0, 1);
	printf("  %7ld Won   ", save_money);

	_delay_ms(3000);
}

void cash_service(void)
{
	id_check();

	lcd_gotoxy(0, 0);
	lcd_string("1.Input/Output  ");
	lcd_gotoxy(0, 1);
	lcd_string("2.Account Check ");

	while( 1 ) {
		key = getkey(key);
		if( key_flag ) {
			if( key == KEY_1 ) {
				inout_mode();
				return;
			}
			else if( key == KEY_2 ) {
				account_mode();
				return;
			}
		}
	}
}

int main(void)
{
	mcu_init();
	fdevopen((void *)lcd_data_write, 0);

	start_bankbook();
	init_screen();

	while( 1 ) {
		key = getkey(key);
		if( key_flag ) {
			switch( key ) {
				case KEY_1 :
					if( user_set )	cash_service();
					init_screen();
					break;

				case KEY_2 :
					if( user_set )	new_user_setting();
					else			user_setting();
					init_screen();
					break;
			}
		}
	}

	return 0;
}
